@@ -19,7 +19,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#define FUSE_USE_VERSION  26
+#define FUSE_USE_VERSION  30
+#define FUSE_DARWIN_ENABLE_EXTENSIONS 0
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -205,77 +206,69 @@ static int get_afc_file_mode(afc_file_mode_t *afc_mode, int flags)
 	return 0;
 }
 
-static int ifuse_getattr(const char *path, struct stat *stbuf)
+static int ifuse_getattr(const char *path, struct stat *stbuf, struct fuse_file_info *fi)
 {
-	int i;
 	int res = 0;
-	char **info = NULL;
+	plist_t info = NULL;
 
 	afc_client_t afc = fuse_get_context()->private_data;
-	afc_error_t ret = afc_get_file_info(afc, path, &info);
+	afc_error_t ret = afc_get_file_info_plist(afc, path, &info);
 
 	memset(stbuf, 0, sizeof(struct stat));
 	if (ret != AFC_E_SUCCESS) {
 		int e = get_afc_error_as_errno(ret);
-		res = -e;
+		return -e;
 	} else if (!info) {
-		res = -1;
-	} else {
-		// get file attributes from info list
-		for (i = 0; info[i]; i += 2) {
-			if (!strcmp(info[i], "st_size")) {
-				stbuf->st_size = atoll(info[i+1]);
-			} else if (!strcmp(info[i], "st_blocks")) {
-				stbuf->st_blocks = atoi(info[i+1]);
-			} else if (!strcmp(info[i], "st_ifmt")) {
-				if (!strcmp(info[i+1], "S_IFREG")) {
-					stbuf->st_mode = S_IFREG;
-				} else if (!strcmp(info[i+1], "S_IFDIR")) {
-					stbuf->st_mode = S_IFDIR;
-				} else if (!strcmp(info[i+1], "S_IFLNK")) {
-					stbuf->st_mode = S_IFLNK;
-				} else if (!strcmp(info[i+1], "S_IFBLK")) {
-					stbuf->st_mode = S_IFBLK;
-				} else if (!strcmp(info[i+1], "S_IFCHR")) {
-					stbuf->st_mode = S_IFCHR;
-				} else if (!strcmp(info[i+1], "S_IFIFO")) {
-					stbuf->st_mode = S_IFIFO;
-				} else if (!strcmp(info[i+1], "S_IFSOCK")) {
-					stbuf->st_mode = S_IFSOCK;
-				}
-			} else if (!strcmp(info[i], "st_nlink")) {
-				stbuf->st_nlink = atoi(info[i+1]);
-			} else if (!strcmp(info[i], "st_mtime")) {
-				stbuf->st_mtime = (time_t)(atoll(info[i+1]) / 1000000000);
-			}
-#ifdef _DARWIN_FEATURE_64_BIT_INODE
-			else if (!strcmp(info[i], "st_birthtime")) { /* available on iOS 7+ */
-				stbuf->st_birthtime = (time_t)(atoll(info[i+1]) / 1000000000);
-			}
-#endif
-		}
-		free_dictionary(info);
-
-		// set permission bits according to the file type
-		if (S_ISDIR(stbuf->st_mode)) {
-			stbuf->st_mode |= 0755;
-		} else if (S_ISLNK(stbuf->st_mode)) {
-			stbuf->st_mode |= 0777;
-		} else {
-			stbuf->st_mode |= 0644;
-		}
-
-		// and set some additional info
-		stbuf->st_uid = getuid();
-		stbuf->st_gid = getgid();
-
-		stbuf->st_blksize = g_blocksize;
+		return -1;
 	}
 
+	stbuf->st_size = plist_dict_get_uint(info, "st_size");
+	stbuf->st_blocks = plist_dict_get_uint(info, "st_blocks");
+	const char* s_ifmt = plist_get_string_ptr(plist_dict_get_item(info, "st_ifmt"), NULL);
+	if (s_ifmt) {
+		if (!strcmp(s_ifmt, "S_IFREG")) {
+			stbuf->st_mode = S_IFREG;
+		} else if (!strcmp(s_ifmt, "S_IFDIR")) {
+			stbuf->st_mode = S_IFDIR;
+		} else if (!strcmp(s_ifmt, "S_IFLNK")) {
+			stbuf->st_mode = S_IFLNK;
+		} else if (!strcmp(s_ifmt, "S_IFBLK")) {
+			stbuf->st_mode = S_IFBLK;
+		} else if (!strcmp(s_ifmt, "S_IFCHR")) {
+			stbuf->st_mode = S_IFCHR;
+		} else if (!strcmp(s_ifmt, "S_IFIFO")) {
+			stbuf->st_mode = S_IFIFO;
+		} else if (!strcmp(s_ifmt, "S_IFSOCK")) {
+			stbuf->st_mode = S_IFSOCK;
+		}
+	}
+	stbuf->st_nlink = plist_dict_get_uint(info, "st_nlink");
+	stbuf->st_mtime = (time_t)(plist_dict_get_uint(info, "st_mtime") / 1000000000);
+#ifdef _DARWIN_FEATURE_64_BIT_INODE
+	/* available on iOS 7+ */
+	stbuf->st_birthtime = (time_t)(plist_dict_get_uint(info, "st_birthtime") / 1000000000);
+#endif
+	plist_free(info);
+
+	// set permission bits according to the file type
+	if (S_ISDIR(stbuf->st_mode)) {
+		stbuf->st_mode |= 0755;
+	} else if (S_ISLNK(stbuf->st_mode)) {
+		stbuf->st_mode |= 0777;
+	} else {
+		stbuf->st_mode |= 0644;
+	}
+
+	// and set some additional info
+	stbuf->st_uid = getuid();
+	stbuf->st_gid = getgid();
+
+	stbuf->st_blksize = g_blocksize;
+
 	return res;
 }
 
-static int ifuse_readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info *fi)
+static int ifuse_readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info *fi, enum fuse_readdir_flags flags)
 {
 	int i;
 	char **dirs = NULL;
@@ -287,7 +280,7 @@ static int ifuse_readdir(const char *path, void *buf, fuse_fill_dir_t filler, of
 		return -ENOENT;
 
 	for (i = 0; dirs[i]; i++) {
-		filler(buf, dirs[i], NULL, 0);
+		filler(buf, dirs[i], NULL, 0, 0);
 	}
 
 	free_dictionary(dirs);
@@ -366,7 +359,7 @@ static int ifuse_write(const char *path, const char *buf, size_t size, off_t off
 	return bytes;
 }
 
-static int ifuse_utimens(const char *path, const struct timespec tv[2])
+static int ifuse_utimens(const char *path, const struct timespec tv[2], struct fuse_file_info *fi)
 {
 	afc_client_t afc = fuse_get_context()->private_data;
 	uint64_t mtime = (uint64_t)tv[1].tv_sec * (uint64_t)1000000000 + (uint64_t)tv[1].tv_nsec;
@@ -398,11 +391,11 @@ static int ifuse_release(const char *path, struct fuse_file_info *fi)
 	return 0;
 }
 
-void *ifuse_init(struct fuse_conn_info *conn)
+void *ifuse_init(struct fuse_conn_info *conn, struct fuse_config *cfg)
 {
 	afc_client_t afc = NULL;
 
-	conn->async_read = 0;
+	conn->want &= FUSE_CAP_ASYNC_READ;
 
 	if (house_arrest) {
 		afc_client_new_from_house_arrest_client(house_arrest, &afc);
@@ -415,16 +408,9 @@ void *ifuse_init(struct fuse_conn_info *conn)
 
 	if (afc) {
 		// get file system block size
-		int i;
-		char **info_raw = NULL;
-		if ((AFC_E_SUCCESS == afc_get_device_info(afc, &info_raw)) && info_raw) {
-			for (i = 0; info_raw[i]; i+=2) {
-				if (!strcmp(info_raw[i], "FSBlockSize")) {
-					g_blocksize = atoi(info_raw[i + 1]);
-					break;
-				}
-			}
-			free_dictionary(info_raw);
+		plist_t info = NULL;
+		if ((AFC_E_SUCCESS == afc_get_device_info_plist(afc, &info)) && info) {
+			g_blocksize = plist_dict_get_uint(info, "FSBlockSize");
 		}
 	}
 
@@ -447,6 +433,16 @@ int ifuse_flush(const char *path, struct fuse_file_info *fi)
 	return 0;
 }
 
+int ifuse_chmod(const char *path, mode_t mode, struct fuse_file_info *fi)
+{
+	return 0;
+}
+
+int ifuse_chown(const char *file, uid_t user, gid_t group, struct fuse_file_info *fi)
+{
+	return 0;
+}
+
 int ifuse_statfs(const char *path, struct statvfs *stats)
 {
 	afc_client_t afc = fuse_get_context()->private_data;
@@ -482,7 +478,7 @@ int ifuse_statfs(const char *path, struct statvfs *stats)
 	return 0;
 }
 
-int ifuse_truncate(const char *path, off_t size)
+int ifuse_truncate(const char *path, off_t size, struct fuse_file_info *fi)
 {
 	afc_client_t afc = fuse_get_context()->private_data;
 	afc_error_t err = afc_truncate(afc, path, size);
@@ -493,18 +489,6 @@ int ifuse_truncate(const char *path, off_t size)
 	return 0;
 }
 
-int ifuse_ftruncate(const char *path, off_t size, struct fuse_file_info *fi)
-{
-	afc_client_t afc = fuse_get_context()->private_data;
-
-	afc_error_t err = afc_file_truncate(afc, fi->fh, size);
-	if (err != AFC_E_SUCCESS) {
-		int res = get_afc_error_as_errno(err);
-		return -res;
-	}
-	return 0;
-}
-
 int ifuse_readlink(const char *path, char *linktarget, size_t buflen)
 {
 	int i, ret;
@@ -566,7 +550,7 @@ int ifuse_unlink(const char *path)
 	return -get_afc_error_as_errno(err);
 }
 
-int ifuse_rename(const char *from, const char *to)
+int ifuse_rename(const char *from, const char *to, unsigned int flags)
 {
 	afc_client_t afc = fuse_get_context()->private_data;
 
@@ -599,7 +583,6 @@ static struct fuse_operations ifuse_oper = {
 	.read = ifuse_read,
 	.write = ifuse_write,
 	.truncate = ifuse_truncate,
-	.ftruncate = ifuse_ftruncate,
 	.readlink = ifuse_readlink,
 	.symlink = ifuse_symlink,
 	.link = ifuse_link,
@@ -607,6 +590,8 @@ static struct fuse_operations ifuse_oper = {
 	.rename = ifuse_rename,
 	.utimens = ifuse_utimens,
 	.fsync = ifuse_fsync,
+	.chmod = ifuse_chmod,
+	.chown = ifuse_chown,
 	.release = ifuse_release,
 	.init = ifuse_init,
 	.destroy = ifuse_cleanup
@@ -820,9 +805,7 @@ int main(int argc, char *argv[])
 		} else {
 			printf("ERROR: No device found!\n");
 		}
-		fprintf(stderr, "Is the device properly connected?\n");
-		fprintf(stderr, "If it is make sure that your user has permissions to access the raw USB device.\n");
-		fprintf(stderr, "If you're still having issues try unplugging the device and reconnecting it.\n");
+		fprintf(stderr, "Ensure that the device is connected and usbmuxd is properly installed.\n");
 		return EXIT_FAILURE;
 	}
 
